@@ -51,7 +51,7 @@ H3D_PARAMETERS = ("Input_Primaries %.6f %.6f %.6f %.6f %.6f %.6f %.6f %.6f\r\n"
 				  "Input_Range 16 235\r\nOutput_Range 16 235\r\n\x00")
 
 
-min_version = (0, 88, 14, 0)
+min_version = (0, 88, 20, 0)
 
 
 # Search for madTPG on the local PC, connect to the first found instance
@@ -71,7 +71,7 @@ CM_Fail = 5
 _methodnames = ("ConnectEx", "Disable3dlut", "Enable3dlut", "EnterFullscreen",
 				"GetBlackAndWhiteLevel", "GetDeviceGammaRamp",
 				"GetSelected3dlut", "GetVersion",
-				"IsDisableOsdButtonPressed", "IsFullscreen",
+				"IsDisableOsdButtonPressed", "IsFseModeEnabled", "IsFullscreen",
 				"IsStayOnTopButtonPressed",
 				"IsUseFullscreenButtonPressed", "LeaveFullscreen",
 				"SetDisableOsdButton",
@@ -172,13 +172,15 @@ def icc_device_link_to_madvr(icc_device_link_filename, unity=False,
 				prevperc = perc
 	else:
 		link = ICCP.ICCProfile(icc_device_link_filename)
+		# Need a worker for abort event handling
+		worker = worker_base.WorkerBase()
 		# icclu verbose=0 gives a speed increase
 		xicclu = worker_base.MP_Xicclu(link, scale=clutmax, use_icclu=True,
 									   logfile=logfile,
 									   output_format=("<H", 65535),
 									   reverse=True, output_stream=raw,
 									   convert_video_rgb_to_clut65=convert_video_rgb_to_clut65,
-									   verbose=0)
+									   verbose=0, worker=worker)
 		xicclu._in = ci.Cube3D(clutres)
 		logfile.write("Looking up 256^3 input values through device link and "
 					  "writing madVR 3D LUT...\n")
@@ -845,6 +847,7 @@ class MadTPG_Net(MadTPGBase):
 						if self.debug:
 							safe_print("MadTPG_Net: Client %s:%i stopped sending" %
 									   addr[:2])
+						send_bye = False
 						break
 					blob += incoming
 					if self.debug:
@@ -861,7 +864,10 @@ class MadTPG_Net(MadTPGBase):
 							if record is None:
 								# Need more data
 								break
-							self._process(record, conn)
+							try:
+								self._process(record, conn)
+							except socket.error, exception:
+								safe_print("MadTPG_Net:", exception)
 		with _lock:
 			self._remove_client(addr, send_bye=addr in self._client_sockets and
 											   send_bye)
@@ -1141,7 +1147,7 @@ class MadTPG_Net(MadTPGBase):
 							if self.debug:
 								safe_print("MadTPG_Net: Preventing duplicate connection %s:%i" %
 										   addr[:2])
-							self._remove_client(addr)
+							self._remove_client(addr, False)
 							return
 				self._dispatch_event("on_client_added", (addr, client))
 			else:
@@ -1172,11 +1178,22 @@ class MadTPG_Net(MadTPGBase):
 			self._remove_client(addr)
 		self._incoming[addr].append((commandno, command, params, component))
 
+	def get_black_and_white_level(self):
+		# XXX: madHcNetXX.dll exports madVR_GetBlackAndWhiteLevel,
+		# but the equivalent madVR network protocol command is
+		# GetBlackWhiteLevel (without the "And")!
+		return MadTPG_Net_Sender(self, self._client_socket, "GetBlackWhiteLevel")()
+
 	def get_version(self):
 		""" Return madVR version """
-		return (self._client_socket and
-				self.clients.get(self._client_socket.getpeername(),
-								 {}).get("mvrVersion") or False)
+		try:
+			return (self._client_socket and
+					self.clients.get(self._client_socket.getpeername(),
+									 {}).get("mvrVersion") or False)
+		except socket.error, exception:
+			if self.debug:
+				safe_print("MadTPG_Net:", exception)
+			return False
 
 	def _assemble_hello_params(self):
 		""" Assemble 'hello' packet parameters """
@@ -1209,7 +1226,11 @@ class MadTPG_Net(MadTPGBase):
 		""" Wait until expected reply or timeout. Return reply params or False. """
 		if not isinstance(params, (list, tuple)):
 			params = (params, )
-		addr = conn.getpeername()
+		try:
+			addr = conn.getpeername()
+		except socket.error, exception:
+			safe_print("MadTPG_Net:", exception)
+			return False
 		start = end = time()
 		while end - start < timeout:
 			for reply in self._incoming.get(addr, []):
@@ -1326,15 +1347,18 @@ class MadTPG_Net(MadTPGBase):
 					params[param] = tuple(values)
 		elif command == "reply":
 			commandno = record["commandNo"]
-			repliedcomamnd = self._commands.get(commandno)
-			if repliedcomamnd:
+			repliedcommand = self._commands.get(commandno)
+			if repliedcommand:
 				self._commands.pop(commandno)
-				if repliedcomamnd == "GetBlackAndWhiteLevel":
+				# XXX: madHcNetXX.dll exports madVR_GetBlackAndWhiteLevel,
+				# but the equivalent madVR network protocol command is
+				# GetBlackWhiteLevel (without the "And")!
+				if repliedcommand == "GetBlackWhiteLevel":
 					if len(params) == 8:
 						params = struct.unpack("<ii", params)
 					else:
 						params = False
-				elif repliedcomamnd == "GetDeviceGammaRamp":
+				elif repliedcommand == "GetDeviceGammaRamp":
 					# Convert to ushort_Array_256_Array_3
 					ramp = ((ctypes.c_ushort * 256) * 3)()
 					if len(params) == 1536:
@@ -1345,12 +1369,12 @@ class MadTPG_Net(MadTPGBase):
 						params = ramp
 					else:
 						params = False
-				elif repliedcomamnd == "GetPatternConfig":
+				elif repliedcommand == "GetPatternConfig":
 					if len(params) == 16:
 						params = struct.unpack("<iiii", params)
 					else:
 						params = False
-				elif repliedcomamnd in ("GetSelected3dlut", ):
+				elif repliedcommand in ("GetSelected3dlut", ):
 					if len(params) == 4:
 						params = struct.unpack("<i", params[0:4])[0]
 					else:
@@ -1450,9 +1474,12 @@ class MadTPG_Net(MadTPGBase):
 
 	@property
 	def uri(self):
-		return "%s:%s" % (self._client_socket and
-						  self._client_socket.getpeername()[:2] or
-						  ("0.0.0.0", 0))
+		try:
+			addr = self._client_socket and self._client_socket.getpeername()[:2]
+		except socket.error, exception:
+			safe_print("MadTPG_Net:", exception)
+			addr = None
+		return "%s:%s" % (addr or ("0.0.0.0", 0))
 
 
 class MadTPG_Net_Sender(object):
